@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2000,2001,2002 Nikos Mavroyanopoulos
+ *  Copyright (C) 2000,2001,2002,2003 Nikos Mavroyanopoulos
  *
  *  This file is part of GNUTLS.
  *
@@ -37,7 +37,7 @@
 extern "C" {
 #endif
 
-#define LIBGNUTLS_VERSION "0.8.12"
+#define LIBGNUTLS_VERSION "0.9.0"
 
 /* For backwards compatibility */
 #include <gnutls/compat4.h>
@@ -116,16 +116,17 @@ typedef enum gnutls_protocol_version { GNUTLS_SSL3=1, GNUTLS_TLS1 } gnutls_proto
 typedef enum gnutls_certificate_type { GNUTLS_CRT_X509=1, GNUTLS_CRT_OPENPGP 
 } gnutls_certificate_type;
 
-typedef enum gnutls_x509_certificate_format { GNUTLS_X509_FMT_DER, 
-	GNUTLS_X509_FMT_PEM } gnutls_x509_certificate_format;
+typedef enum gnutls_x509_crt_fmt { GNUTLS_X509_FMT_DER, 
+	GNUTLS_X509_FMT_PEM } gnutls_x509_crt_fmt;
 
-typedef enum gnutls_pk_algorithm { GNUTLS_PK_RSA = 1, GNUTLS_PK_DSA
+typedef enum gnutls_pk_algorithm { GNUTLS_PK_RSA = 1, GNUTLS_PK_DSA,
+	GNUTLS_PK_UNKNOWN = 0xff
 } gnutls_pk_algorithm;
 
 /* If you want to change this, then also change the 
  * define in gnutls_int.h, and recompile.
  */
-typedef void * gnutls_transport_ptr;
+typedef int gnutls_transport_ptr;
 
 struct gnutls_session_int;
 typedef struct gnutls_session_int* gnutls_session;
@@ -304,26 +305,28 @@ int gnutls_certificate_set_dh_params(gnutls_certificate_credentials res, gnutls_
 int gnutls_certificate_set_rsa_params(gnutls_certificate_credentials res, gnutls_rsa_params rsa_params);
 
 int gnutls_certificate_set_x509_trust_file( gnutls_certificate_credentials res, const char* CAFILE, 
-	gnutls_x509_certificate_format);
+	gnutls_x509_crt_fmt);
 int gnutls_certificate_set_x509_trust_mem(gnutls_certificate_credentials res, 
-	const gnutls_datum *CA, gnutls_x509_certificate_format);
+	const gnutls_datum *CA, gnutls_x509_crt_fmt);
+
+int gnutls_certificate_set_x509_crl_file(gnutls_certificate_credentials res, 
+		const char *crlfile, gnutls_x509_crt_fmt type);
+int gnutls_certificate_set_x509_crl_mem(gnutls_certificate_credentials res, 
+	const gnutls_datum *CRL, gnutls_x509_crt_fmt type);
 
 int gnutls_certificate_set_x509_key_file( gnutls_certificate_credentials res, 
-	const char *CERTFILE, const char* KEYFILE, gnutls_x509_certificate_format);
+	const char *CERTFILE, const char* KEYFILE, gnutls_x509_crt_fmt);
 int gnutls_certificate_set_x509_key_mem(gnutls_certificate_credentials res, 
 	const gnutls_datum* CERT, const gnutls_datum* KEY,
-	gnutls_x509_certificate_format);
+	gnutls_x509_crt_fmt);
 
 /* global state functions 
  */
-/* In this version global_init accepts two files (pkix.asn, pkcs1.asn).
- * This will not be the case in the final version. These files 
- * are located in the src/ directory of gnutls distribution.
- */
 int gnutls_global_init(void);
 void gnutls_global_deinit(void);
 
 typedef void* (*gnutls_alloc_function)(size_t);
+typedef void* (*gnutls_calloc_function)(size_t, size_t);
 typedef void (*gnutls_free_function)(void*);
 typedef void* (*gnutls_realloc_function)(void*, size_t);
 
@@ -334,29 +337,42 @@ void gnutls_global_set_mem_functions(
 
 /* For use in callbacks */
 extern gnutls_alloc_function gnutls_malloc;
+extern gnutls_calloc_function gnutls_calloc;
 extern gnutls_free_function gnutls_free;
 
 typedef void (*gnutls_log_func)( const char*);
 void gnutls_global_set_log_function( gnutls_log_func log_func);
 
-/* DH params */
-int gnutls_dh_params_set( gnutls_dh_params, gnutls_datum prime, gnutls_datum generator, int bits);
+/* Diffie Hellman parameter handling.
+ */
 int gnutls_dh_params_init( gnutls_dh_params*);
 void gnutls_dh_params_deinit( gnutls_dh_params);
-int gnutls_dh_params_generate( gnutls_datum* prime, gnutls_datum* generator, int bits);
+int gnutls_dh_params_import_raw(gnutls_dh_params dh_params, const gnutls_datum *prime,
+			const gnutls_datum* generator);
+int gnutls_dh_params_import_pkcs3(gnutls_dh_params params, 
+	const gnutls_datum * pkcs3_params, gnutls_x509_crt_fmt format);
+int gnutls_dh_params_generate2(gnutls_dh_params params, int bits);
+int gnutls_dh_params_export_pkcs3( gnutls_dh_params params,
+	   gnutls_x509_crt_fmt format, unsigned char* params_data, int* params_data_size);
+int gnutls_dh_params_export_raw(gnutls_dh_params params,
+	gnutls_datum * prime, gnutls_datum * generator, int *bits);
 
-/* RSA params */
-int gnutls_rsa_params_set(gnutls_rsa_params rsa_params, 
-	gnutls_datum m, gnutls_datum e, gnutls_datum d, 
-	gnutls_datum p, gnutls_datum q, gnutls_datum u,
-	int bits);
-int gnutls_rsa_params_generate(gnutls_datum * m, gnutls_datum *e,
-	gnutls_datum *d, gnutls_datum *p, gnutls_datum* q, 
-	gnutls_datum* u, int bits);
+
+/* RSA params 
+ */
 int gnutls_rsa_params_init(gnutls_rsa_params * rsa_params);
 void gnutls_rsa_params_deinit(gnutls_rsa_params rsa_params);
+int gnutls_rsa_params_import_raw(gnutls_rsa_params rsa_params, 
+	gnutls_datum m, gnutls_datum e,
+	gnutls_datum d, gnutls_datum p, gnutls_datum q, gnutls_datum u);
+int gnutls_rsa_params_generate2(gnutls_rsa_params params, int bits);
+int gnutls_rsa_params_export_raw(gnutls_rsa_params params,
+	gnutls_datum * m, gnutls_datum *e,
+	gnutls_datum *d, gnutls_datum *p, gnutls_datum* q, 
+	gnutls_datum* u, int *bits);
 
-
+/* Session stuff
+ */
 typedef ssize_t (*gnutls_pull_func)(gnutls_transport_ptr, void*, size_t);
 typedef ssize_t (*gnutls_push_func)(gnutls_transport_ptr, const void*, size_t);
 void gnutls_transport_set_ptr(gnutls_session session, gnutls_transport_ptr ptr);
@@ -381,38 +397,13 @@ void* gnutls_session_get_ptr(gnutls_session session);
 
 void gnutls_openpgp_send_key(gnutls_session session, gnutls_openpgp_key_status status);
 
-int gnutls_x509_fingerprint(gnutls_digest_algorithm algo, const gnutls_datum* data, char* result, size_t* result_size);
-
+/* fingerprint */
+int gnutls_fingerprint(gnutls_digest_algorithm algo, const gnutls_datum* data, 
+	char* result, size_t* result_size);
 
 #ifndef GNUTLS_UI_H
 # define GNUTLS_UI_H
 
-
-/* Extra definitions */
-
-#define GNUTLS_X509_CN_SIZE 256
-#define GNUTLS_X509_C_SIZE 3
-#define GNUTLS_X509_O_SIZE 256
-#define GNUTLS_X509_OU_SIZE 256
-#define GNUTLS_X509_L_SIZE 256
-#define GNUTLS_X509_S_SIZE 256
-#define GNUTLS_X509_EMAIL_SIZE 256
-
-typedef struct {
-	char common_name[GNUTLS_X509_CN_SIZE];
-	char country[GNUTLS_X509_C_SIZE];
-	char organization[GNUTLS_X509_O_SIZE];
-	char organizational_unit_name[GNUTLS_X509_OU_SIZE];
-	char locality_name[GNUTLS_X509_L_SIZE];
-	char state_or_province_name[GNUTLS_X509_S_SIZE];
-	char email[GNUTLS_X509_EMAIL_SIZE];
-} gnutls_x509_dn;
-
-typedef struct {
-	char name[GNUTLS_X509_CN_SIZE];
-	char email[GNUTLS_X509_CN_SIZE];
-} gnutls_openpgp_name;	
-
 typedef enum gnutls_x509_subject_alt_name {
 	GNUTLS_SAN_DNSNAME=1, GNUTLS_SAN_RFC822NAME,
 	GNUTLS_SAN_URI, GNUTLS_SAN_IPADDRESS
@@ -420,10 +411,14 @@ typedef enum gnutls_x509_subject_alt_name {
 
 # ifdef LIBGNUTLS_VERSION /* These are defined only in gnutls.h */
 
-typedef int gnutls_certificate_client_select_function(gnutls_session, 
-	const gnutls_datum *, unsigned int, const gnutls_datum *, unsigned int);
-typedef int gnutls_certificate_server_select_function(gnutls_session, 
-	const gnutls_datum *, unsigned int);
+/* Callback prototypes for the certificate authentication
+ * callbacks.
+ */
+typedef int gnutls_certificate_client_select_function(
+   gnutls_session, const gnutls_datum *client_cert, int ncerts, 
+   const gnutls_datum* req_ca_cert, int nreqs);
+typedef int gnutls_certificate_server_select_function(
+   gnutls_session, const gnutls_datum *server_certs, int ncerts);
 
 /* Functions that allow AUTH_INFO structures handling
  */
@@ -442,37 +437,25 @@ int gnutls_rsa_export_get_modulus_bits(gnutls_session session);
 
 /* X509PKI */
 
+#include <gnutls/compat8.h>
+
 void gnutls_certificate_client_set_select_function( gnutls_session, gnutls_certificate_client_select_function *);
 void gnutls_certificate_server_set_select_function( gnutls_session, gnutls_certificate_server_select_function *);
 
 void gnutls_certificate_server_set_request( gnutls_session, gnutls_certificate_request);
 
-/* X.509 certificate handling functions */
-int gnutls_x509_certificate_to_xml(const gnutls_datum * cert, gnutls_datum* res, int detail);
+/* X.509 certificate handling functions 
+ */
 
-int gnutls_x509_extract_dn( const gnutls_datum*, gnutls_x509_dn*);
-int gnutls_x509_extract_dn_string(const gnutls_datum * idn,
-        char *buf, unsigned int sizeof_buf);
-int gnutls_x509_extract_certificate_dn( const gnutls_datum*, gnutls_x509_dn*);
-int gnutls_x509_extract_certificate_dn_string(char *buf, unsigned int sizeof_buf,
-   const gnutls_datum * cert, int issuer);
-int gnutls_x509_extract_certificate_issuer_dn(  const gnutls_datum*, gnutls_x509_dn *);
-int gnutls_x509_extract_certificate_version( const gnutls_datum*);
-int gnutls_x509_extract_certificate_serial(const gnutls_datum * cert, char* result, int* result_size);
-time_t gnutls_x509_extract_certificate_activation_time( const gnutls_datum*);
-time_t gnutls_x509_extract_certificate_expiration_time( const gnutls_datum*);
-int gnutls_x509_extract_certificate_subject_alt_name( const gnutls_datum*, int seq, char*, int*);
-int gnutls_x509_pkcs7_extract_certificate(const gnutls_datum * pkcs7_struct, int indx, char* certificate, int* certificate_size);
-int gnutls_x509_extract_certificate_pk_algorithm( const gnutls_datum * cert, int* bits);
-int gnutls_x509_extract_certificate_ca_status(const gnutls_datum * cert);
-int gnutls_x509_extract_key_pk_algorithm( const gnutls_datum * key);
+int gnutls_pkcs3_extract_dh_params(const gnutls_datum * params,
+	gnutls_x509_crt_fmt format, gnutls_datum * prime,
+	gnutls_datum * generator, int* prime_bits);
+int gnutls_pkcs3_export_dh_params( const gnutls_datum * prime,
+	const gnutls_datum * generator, gnutls_x509_crt_fmt format,
+	unsigned char* params_data, int* params_data_size);
 
-int gnutls_x509_verify_certificate( const gnutls_datum* cert_list, int cert_list_length, const gnutls_datum * CA_list, int CA_list_length, const gnutls_datum* CRL_list, int CRL_list_length);
-
-int gnutls_x509_check_certificates_hostname(const gnutls_datum * cert,
-                                const char *hostname);
-
-/* get data from the session */
+/* get data from the session 
+ */
 const gnutls_datum* gnutls_certificate_get_peers( gnutls_session, int* list_size);
 const gnutls_datum *gnutls_certificate_get_ours( gnutls_session session);
 
@@ -492,6 +475,10 @@ int gnutls_pem_base64_decode_alloc( const char* header, const gnutls_datum *b64_
 
 #endif /* GNUTLS_UI_H */
 
+
+#ifndef GNUTLS_ERRORS_IH
+# define GNUTLS_ERRORS_IH
+
 /* Gnutls error codes. The mapping to a TLS alert is also shown in
  * comments.
  */
@@ -591,11 +578,15 @@ int gnutls_pem_base64_decode_alloc( const char* header, const gnutls_datum *b64_
 
 #define GNUTLS_E_ILLEGAL_SRP_USERNAME -90
 #define GNUTLS_E_SRP_PWD_PARSING_ERROR -91
+#define GNUTLS_E_EMPTY_SRP_USERNAME -92
+#define GNUTLS_E_NO_TEMPORARY_DH_PARAMS -93
 
 #define GNUTLS_E_UNIMPLEMENTED_FEATURE -250
 
 
 
+#endif /* GNUTLS_ERRORS_IH */
+
 #ifdef __cplusplus
 }
 #endif
