@@ -105,10 +105,9 @@ static int gen_dhe_server_kx(gnutls_session session, opaque ** data)
 		return ret;
 	}
 
-	g = gnutls_get_dh_params( cred->dh_params, &p, bits);
-	if (g == NULL) {
+	if ( (ret=_gnutls_get_dh_params( cred->dh_params, &p, &g)) < 0) {
 		gnutls_assert();
-		return GNUTLS_E_MEMORY_ERROR;
+		return ret;
 	}
 
 	if ( (ret=_gnutls_auth_info_set( session, GNUTLS_CRD_CERTIFICATE, 
@@ -210,7 +209,7 @@ static int proc_dhe_server_kx(gnutls_session session, opaque * data,
 		case GNUTLS_CRT_X509:
 			if ((ret =
 			     _gnutls_x509_cert2gnutls_cert( &peer_cert,
-					     info->raw_certificate_list[0], CERT_NO_COPY)) < 0) {
+					     &info->raw_certificate_list[0], CERT_NO_COPY)) < 0) {
 				gnutls_assert();
 				return ret;
 			}
@@ -223,7 +222,7 @@ static int proc_dhe_server_kx(gnutls_session session, opaque * data,
 			}
 			if ((ret =
 			     _E_gnutls_openpgp_cert2gnutls_cert( &peer_cert,
-					     info->raw_certificate_list[0])) < 0) {
+					     &info->raw_certificate_list[0])) < 0) {
 				gnutls_assert();
 				return ret;
 			}
@@ -239,7 +238,7 @@ static int proc_dhe_server_kx(gnutls_session session, opaque * data,
 				      &peer_cert,
 				      &vparams, &signature);
 	
-	_gnutls_free_cert( peer_cert);
+	_gnutls_free_cert( &peer_cert);
 	if (ret < 0) {
 		gnutls_assert();
 		return ret;
@@ -266,10 +265,9 @@ GNUTLS_MPI p, g;
 	        return GNUTLS_E_INSUFICIENT_CREDENTIALS;
 	}
 
-	g = gnutls_get_dh_params( cred->dh_params, &p, bits);
-	if (g == NULL || p == NULL) {
+	if ( (ret=_gnutls_get_dh_params( cred->dh_params, &p, &g)) < 0) {
 		gnutls_assert();
-		return GNUTLS_E_MEMORY_ERROR;
+		return ret;
 	}
 
 	ret = _gnutls_proc_dh_common_client_kx( session, data, _data_size, g, p);
