@@ -35,7 +35,6 @@
 #include <gnutls_pk.h>
 #include <gnutls_algorithms.h>
 #include <gnutls_global.h>
-#include <x509_verify.h>
 #include "debug.h"
 #include <gnutls_sig.h>
 #include <gnutls_x509.h>
@@ -86,7 +85,7 @@ int i;
 		case GNUTLS_CRT_X509:
 			if ((ret =
 			     _gnutls_x509_cert2gnutls_cert( &peer_cert,
-					     info->raw_certificate_list[0], CERT_ONLY_PUBKEY|CERT_NO_COPY)) < 0) {
+					     &info->raw_certificate_list[0], CERT_ONLY_PUBKEY|CERT_NO_COPY)) < 0) {
 				gnutls_assert();
 				return ret;
 			}
@@ -99,7 +98,7 @@ int i;
 			}
 			if ((ret =
 			     _E_gnutls_openpgp_cert2gnutls_cert( &peer_cert,
-					     info->raw_certificate_list[0])) < 0) {
+					     &info->raw_certificate_list[0])) < 0) {
 				gnutls_assert();
 				return ret;
 			}
@@ -116,7 +115,7 @@ int i;
 		 == GNUTLS_KX_RSA_EXPORT && 
 		 	_gnutls_mpi_get_nbits(peer_cert.params[0]) > 512) {
 
-		_gnutls_free_cert( peer_cert);
+		_gnutls_free_cert( &peer_cert);
 
 		if (session->key->rsa[0] == NULL ||
 			session->key->rsa[1] == NULL) {
@@ -147,7 +146,7 @@ int i;
 	for (i=0;i<*params_len;i++) {
 		params[i] = _gnutls_mpi_copy(peer_cert.params[i]);
 	}
-	_gnutls_free_cert( peer_cert);
+	_gnutls_free_cert( &peer_cert);
 
 	return 0;
 }
@@ -214,7 +213,6 @@ int _gnutls_proc_rsa_client_kx(gnutls_session session, opaque * data, size_t _da
 	int ret, dsize;
 	GNUTLS_MPI *params;
 	int params_len;
-	int randomize_key = 0;
 	ssize_t data_size = _data_size;
 
 	if (gnutls_protocol_get_version(session) == GNUTLS_SSL3) {
@@ -243,47 +241,32 @@ int _gnutls_proc_rsa_client_kx(gnutls_session session, opaque * data, size_t _da
 		params_len, 2);	/* btype==2 */
 
 	if (ret < 0 || plaintext.size != TLS_MASTER_SIZE) {
-		/* In case decryption fails then don't inform
+		/* in case decryption fails then don't inform
 		 * the peer. Just use a random key. (in order to avoid
 		 * attack against pkcs-1 formating).
 		 */
+		ret = 0;
 		gnutls_assert();
-		_gnutls_x509_log("auth_rsa: Possible PKCS #1 format attack\n");
-		randomize_key = 1;
-	} else {
-		/* If the secret was properly formatted, then
-		 * check the version number.
-		 */
-		if (_gnutls_get_adv_version_major(session) != plaintext.data[0]
-		    || _gnutls_get_adv_version_minor(session) != plaintext.data[1]) 
-		{
-		    	/* No error is returned here, if the version number check
-		    	 * fails. We proceed normally.
-		    	 * That is to defend against the attack described in the paper
-		    	 * "Attacking RSA-based sessions in SSL/TLS" by Vlastimil Klima,
-		    	 * Ondej Pokorny and Tomas Rosa.
-		    	 */
-			gnutls_assert();
-			_gnutls_x509_log("auth_rsa: Possible PKCS #1 version check format attack\n");
-		}
-	}
+		_gnutls_log("auth_rsa: Possible PKCS-1 format attack\n");
 
-	if (randomize_key != 0) {
 		RANDOMIZE_KEY(session->key->key,
-			      gnutls_malloc, GNUTLS_STRONG_RANDOM);
-
+			      gnutls_secure_malloc, GNUTLS_WEAK_RANDOM);
 	} else {
+		ret = 0;
+		if (session->internals.rsa_pms_check==0)
+			if (_gnutls_get_adv_version_major(session) !=
+			    plaintext.data[0]
+			    || _gnutls_get_adv_version_minor(session) !=
+			    plaintext.data[1]) {
+				gnutls_assert();
+				ret = GNUTLS_E_DECRYPTION_FAILED;
+			}
+
 		session->key->key.data = plaintext.data;
 		session->key->key.size = plaintext.size;
 	}
 
-	/* This is here to avoid the version check attack
-	 * discussed above.
-	 */
-	session->key->key.data[0] = _gnutls_get_adv_version_major(session);
-	session->key->key.data[1] = _gnutls_get_adv_version_minor(session);
-
-	return 0;
+	return ret;
 }
 
 
