@@ -52,6 +52,7 @@ static const gnutls_alert_entry sup_alerts[] = {
 	{ GNUTLS_A_PROTOCOL_VERSION,	 "Error in protocol version" },
 	{ GNUTLS_A_INSUFFICIENT_SECURITY,"Insufficient security" },
 	{ GNUTLS_A_USER_CANCELED,	 "User canceled" },
+	{ GNUTLS_A_INTERNAL_ERROR,	 "Internal error" },
 	{ GNUTLS_A_NO_RENEGOTIATION,	 "No renegotiation is allowed" },
 	{ GNUTLS_A_CERTIFICATE_UNOBTAINABLE, "Could not retrieve the specified certificate" },
 	{ GNUTLS_A_UNSUPPORTED_EXTENSION, "An unsupported extension was sent" },
@@ -135,16 +136,26 @@ int _level = -1;
 	switch (err) { /* send appropriate alert */
 		case GNUTLS_E_DECRYPTION_FAILED:
 			/* GNUTLS_A_DECRYPTION_FAILED is not sent, because
-			 * it is not defined in SSL3.
+			 * it is not defined in SSL3. Note that we must
+			 * not distinguish Decryption failures from mac
+			 * check failures, due to the possibility of some 
+			 * attacks.
 			 */
 			ret = GNUTLS_A_BAD_RECORD_MAC;
 			_level = GNUTLS_AL_FATAL;
 			break;
+		case GNUTLS_E_EMPTY_SRP_USERNAME:
+			/* FIXME: needs to be changed
+			 */
+			ret = GNUTLS_A_ACCESS_DENIED;
+			_level = GNUTLS_AL_FATAL;
+			break;
 		case GNUTLS_E_DECOMPRESSION_FAILED:
 			ret = GNUTLS_A_DECOMPRESSION_FAILURE;
 			_level = GNUTLS_AL_FATAL;
 			break;
 		case GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER:
+		case GNUTLS_E_ILLEGAL_SRP_USERNAME:
                         ret = GNUTLS_A_ILLEGAL_PARAMETER;
 			_level = GNUTLS_AL_FATAL;
                         break;
@@ -193,6 +204,8 @@ int _level = -1;
 			_level = GNUTLS_AL_FATAL;
 			break;
 		case GNUTLS_E_INTERNAL_ERROR:
+		case GNUTLS_E_NO_TEMPORARY_DH_PARAMS:
+		case GNUTLS_E_NO_TEMPORARY_RSA_PARAMS:
 			ret = GNUTLS_A_INTERNAL_ERROR;
 			_level = GNUTLS_AL_FATAL;
 			break;
