@@ -63,6 +63,7 @@ static gnutls_error_entry error_algorithms[] = {
 	ERROR_ENTRY("The peer did not send any certificate.", GNUTLS_E_NO_CERTIFICATE_FOUND, 1 ), 
 
 	ERROR_ENTRY("No temporary RSA parameters were found.", GNUTLS_E_NO_TEMPORARY_RSA_PARAMS, 1 ), 
+	ERROR_ENTRY("No temporary DH parameters were found.", GNUTLS_E_NO_TEMPORARY_DH_PARAMS, 1 ), 
 	ERROR_ENTRY("An unexpected TLS handshake packet was received.", GNUTLS_E_UNEXPECTED_HANDSHAKE_PACKET, 1 ), 
 	ERROR_ENTRY("The scanning of a large integer has failed.", GNUTLS_E_MPI_SCAN_FAILED, 1 ), 
 	ERROR_ENTRY("Could not export a large integer.", GNUTLS_E_MPI_PRINT_FAILED, 1 ), 
@@ -128,6 +129,7 @@ static gnutls_error_entry error_algorithms[] = {
         ERROR_ENTRY("Could not get OpenPGP key.", GNUTLS_E_OPENPGP_GETKEY_FAILED, 1),
 
         ERROR_ENTRY("The SRP username supplied by the peer is illegal.", GNUTLS_E_ILLEGAL_SRP_USERNAME, 1),
+        ERROR_ENTRY("The peer advertized SRP but did not supply any SRP username.", GNUTLS_E_EMPTY_SRP_USERNAME, 1),
 	{0, 0, 0, 0}
 };
 
@@ -249,7 +251,6 @@ int _gnutls_asn2err( int asn_err) {
 /* this function will output a message using the
  * caller provided function 
  */
-#ifdef DEBUG
 void _gnutls_log( const char *fmt, ...) {
  va_list args;
  char str[MAX_LOG_SIZE];
@@ -265,14 +266,13 @@ void _gnutls_log( const char *fmt, ...) {
 
  return;
 }
-#else /* not DEBUG */
+#ifndef DEBUG
 # ifndef C99_MACROS
 
 /* Without C99 macros these functions have to
  * be called. This may affect performance.
  */
 void _gnutls_null_log( void* x, ...) { return; }
-char* GET_CN( gnutls_datum x) { return NULL; }
 const char* _gnutls_handshake2str( int handshake) { return NULL; }
 char * _gnutls_bin2hex(const unsigned char *old, const size_t oldlen)
 	{ return NULL; }
