@@ -1,7 +1,7 @@
 /*
- * Copyright (C) 2001,2002,2003 Nikos Mavroyanopoulos
+ *  Copyright (C) 2001,2002,2003 Nikos Mavroyanopoulos
  *
- * This file is part of GNUTLS.
+ *  This file is part of GNUTLS.
  *
  *  The GNUTLS library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public   
@@ -61,12 +61,12 @@ void gnutls_global_set_log_function( GNUTLS_LOG_FUNC log_func) {
 }
 
 
+#ifdef DEBUG
 /* default logging function */
 static void dlog( const char* str) {
-#ifdef DEBUG
 	fputs( str, stderr);
-#endif
 }
+#endif
 
 extern gnutls_alloc_function gnutls_secure_malloc;
 extern gnutls_alloc_function gnutls_malloc;
@@ -167,15 +167,18 @@ int gnutls_global_init( void)
 		gcry_control (GCRYCTL_INITIALIZATION_FINISHED, NULL,0);
 
 #ifdef DEBUG
+		/* applications may want to override that, so we only use
+		 * it in debugging mode.
+		 */
 		gcry_set_log_handler( _gnutls_gcry_log_handler, NULL);
-#else
-		gcry_control (GCRYCTL_SET_VERBOSITY, (int)0);
 #endif
 	}
 	
 	/* set default recv/send functions
 	 */
+#ifdef DEBUG
 	gnutls_global_set_log_function( dlog);
+#endif
 
 	/* initialize parser 
 	 * This should not deal with files in the final
@@ -193,13 +196,6 @@ int gnutls_global_init( void)
 		return _gnutls_asn2err(result);
 	}
 
-	result = _gnutls_dh_calc_mpis();
-	if (result < 0) {
-		gnutls_assert();
-		return result;
-	}
-
-	
 	return 0;
 }
 
@@ -218,8 +214,6 @@ void gnutls_global_deinit( void) {
 	if (_gnutls_init==0) {
 		asn1_delete_structure(& GNUTLS_ASN);
 		asn1_delete_structure(& PKIX1_ASN);
-	
-		_gnutls_dh_clear_mpis();
 	}
 	
 }
