@@ -6,6 +6,8 @@
 
 #define GNUTLS_MPI GCRY_MPI
 
+#define _gnutls_mpi_cmp_ui gcry_mpi_cmp_ui
+#define _gnutls_mpi_mod gcry_mpi_mod
 #define _gnutls_mpi_new gcry_mpi_new
 #define _gnutls_mpi_snew gcry_mpi_snew
 #define _gnutls_mpi_copy gcry_mpi_copy
@@ -32,9 +34,7 @@ int _gnutls_mpi_scan_pgp( GNUTLS_MPI *ret_mpi, const opaque *buffer, size_t *nby
 int _gnutls_mpi_print( opaque *buffer, size_t *nbytes, const GNUTLS_MPI a );
 int _gnutls_mpi_print_lz( opaque *buffer, size_t *nbytes, const GNUTLS_MPI a );
 
-int _gnutls_x509_read_int( ASN1_TYPE node, const char* value, char* tmpstr, int tmpstr_size, MPI* ret_mpi);
-
-asn1_retCode _gnutls_asn1_create_element(ASN1_TYPE definitions,const char *source_name,
-                                 ASN1_TYPE *element, const char *dest_name);
+int _gnutls_x509_read_int( ASN1_TYPE node, const char* value, 
+	char* tmpstr, int tmpstr_size, GNUTLS_MPI* ret_mpi);
 
 #endif
