@@ -47,9 +47,9 @@ int _gnutls_PKCS1key2gnutlsKey(gnutls_private_key * pkey,
 	pkey->pk_algorithm = GNUTLS_PK_RSA;
 
 	if ((result =
-	     _gnutls_asn1_create_element(_gnutls_get_gnutls_asn(),
-				   "GNUTLS.RSAPrivateKey", &pkey_asn,
-				   "rsakey")) != ASN1_SUCCESS) {
+	     asn1_create_element(_gnutls_get_gnutls_asn(),
+				   "GNUTLS.RSAPrivateKey", &pkey_asn
+				   )) != ASN1_SUCCESS) {
 		gnutls_assert();
 		return _gnutls_asn2err(result);
 	}
@@ -66,7 +66,7 @@ int _gnutls_PKCS1key2gnutlsKey(gnutls_private_key * pkey,
 		return _gnutls_asn2err(result);
 	}
 
-	if ((result = _gnutls_x509_read_int(pkey_asn, "rsakey.modulus",
+	if ((result = _gnutls_x509_read_int(pkey_asn, "modulus",
 					    str, sizeof(str) - 1,
 					    &pkey->params[0])) < 0) {
 		gnutls_assert();
@@ -75,7 +75,7 @@ int _gnutls_PKCS1key2gnutlsKey(gnutls_private_key * pkey,
 	}
 
 	if ((result =
-	     _gnutls_x509_read_int(pkey_asn, "rsakey.publicExponent", str,
+	     _gnutls_x509_read_int(pkey_asn, "publicExponent", str,
 				   sizeof(str) - 1,
 				   &pkey->params[1])) < 0) {
 		gnutls_assert();
@@ -85,7 +85,7 @@ int _gnutls_PKCS1key2gnutlsKey(gnutls_private_key * pkey,
 	}
 
 	if ((result =
-	     _gnutls_x509_read_int(pkey_asn, "rsakey.privateExponent", str,
+	     _gnutls_x509_read_int(pkey_asn, "privateExponent", str,
 				   sizeof(str) - 1,
 				   &pkey->params[2])) < 0) {
 		gnutls_assert();
@@ -95,7 +95,7 @@ int _gnutls_PKCS1key2gnutlsKey(gnutls_private_key * pkey,
 		return result;
 	}
 
-	if ((result = _gnutls_x509_read_int(pkey_asn, "rsakey.prime1",
+	if ((result = _gnutls_x509_read_int(pkey_asn, "prime1",
 					    str, sizeof(str) - 1,
 					    &pkey->params[3])) < 0) {
 		gnutls_assert();
@@ -106,7 +106,7 @@ int _gnutls_PKCS1key2gnutlsKey(gnutls_private_key * pkey,
 		return result;
 	}
 
-	if ((result = _gnutls_x509_read_int(pkey_asn, "rsakey.prime2",
+	if ((result = _gnutls_x509_read_int(pkey_asn, "prime2",
 					    str, sizeof(str) - 1,
 					    &pkey->params[4])) < 0) {
 		gnutls_assert();
@@ -133,7 +133,7 @@ int _gnutls_PKCS1key2gnutlsKey(gnutls_private_key * pkey,
 	_gnutls_mpi_invm(pkey->params[5], pkey->params[3], pkey->params[4]);
 	/*				p, q */
 #else
-	if ( (result=_gnutls_x509_read_int( pkey_asn, "rsakey.coefficient",
+	if ( (result=_gnutls_x509_read_int( pkey_asn, "coefficient",
 		str, sizeof(str)-1, &pkey->params[5])) < 0) {
 		gnutls_assert();
 		_gnutls_mpi_release( &pkey->params[0]);
@@ -150,17 +150,6 @@ int _gnutls_PKCS1key2gnutlsKey(gnutls_private_key * pkey,
 
 	asn1_delete_structure(&pkey_asn);
 
-	if (_gnutls_set_datum(&pkey->raw, raw_key.data, raw_key.size) < 0) {
-		_gnutls_mpi_release(&pkey->params[0]);
-		_gnutls_mpi_release(&pkey->params[1]);
-		_gnutls_mpi_release(&pkey->params[2]);
-		_gnutls_mpi_release(&pkey->params[3]);
-		_gnutls_mpi_release(&pkey->params[4]);
-		_gnutls_mpi_release(&pkey->params[5]);
-		gnutls_assert();
-		return GNUTLS_E_MEMORY_ERROR;
-	}
-
 	return 0;
 
 
@@ -176,9 +165,9 @@ int _gnutls_DSAkey2gnutlsKey(gnutls_private_key * pkey,
 	pkey->pk_algorithm = GNUTLS_PK_DSA;
 
 	if ((result =
-	     _gnutls_asn1_create_element(_gnutls_get_gnutls_asn(),
-				   "GNUTLS.DSAPrivateKey", &dsa_asn,
-				   "dsakey")) != ASN1_SUCCESS) {
+	     asn1_create_element(_gnutls_get_gnutls_asn(),
+				   "GNUTLS.DSAPrivateKey", &dsa_asn
+				   )) != ASN1_SUCCESS) {
 		gnutls_assert();
 		return _gnutls_asn2err(result);
 	}
@@ -195,7 +184,7 @@ int _gnutls_DSAkey2gnutlsKey(gnutls_private_key * pkey,
 		return _gnutls_asn2err(result);
 	}
 
-	if ((result = _gnutls_x509_read_int(dsa_asn, "dsakey.p",
+	if ((result = _gnutls_x509_read_int(dsa_asn, "p",
 					    str, sizeof(str) - 1,
 					    &pkey->params[0])) < 0) {
 		gnutls_assert();
@@ -203,7 +192,7 @@ int _gnutls_DSAkey2gnutlsKey(gnutls_private_key * pkey,
 		return result;
 	}
 
-	if ((result = _gnutls_x509_read_int(dsa_asn, "dsakey.q",
+	if ((result = _gnutls_x509_read_int(dsa_asn, "q",
 					    str, sizeof(str) - 1,
 					    &pkey->params[1])) < 0) {
 		gnutls_assert();
@@ -212,7 +201,7 @@ int _gnutls_DSAkey2gnutlsKey(gnutls_private_key * pkey,
 		return result;
 	}
 
-	if ((result = _gnutls_x509_read_int(dsa_asn, "dsakey.g",
+	if ((result = _gnutls_x509_read_int(dsa_asn, "g",
 					    str, sizeof(str) - 1,
 					    &pkey->params[2])) < 0) {
 		gnutls_assert();
@@ -222,7 +211,7 @@ int _gnutls_DSAkey2gnutlsKey(gnutls_private_key * pkey,
 		return result;
 	}
 
-	if ((result = _gnutls_x509_read_int(dsa_asn, "dsakey.Y",
+	if ((result = _gnutls_x509_read_int(dsa_asn, "Y",
 					    str, sizeof(str) - 1,
 					    &pkey->params[3])) < 0) {
 		gnutls_assert();
@@ -233,7 +222,7 @@ int _gnutls_DSAkey2gnutlsKey(gnutls_private_key * pkey,
 		return result;
 	}
 
-	if ((result = _gnutls_x509_read_int(dsa_asn, "dsakey.priv",
+	if ((result = _gnutls_x509_read_int(dsa_asn, "priv",
 					    str, sizeof(str) - 1,
 					    &pkey->params[4])) < 0) {
 		gnutls_assert();
@@ -248,16 +237,6 @@ int _gnutls_DSAkey2gnutlsKey(gnutls_private_key * pkey,
 
 	asn1_delete_structure(&dsa_asn);
 
-	if (_gnutls_set_datum(&pkey->raw, raw_key.data, raw_key.size) < 0) {
-		_gnutls_mpi_release(&pkey->params[0]);
-		_gnutls_mpi_release(&pkey->params[1]);
-		_gnutls_mpi_release(&pkey->params[2]);
-		_gnutls_mpi_release(&pkey->params[3]);
-		_gnutls_mpi_release(&pkey->params[4]);
-		gnutls_assert();
-		return GNUTLS_E_MEMORY_ERROR;
-	}
-
 	return 0;
 
 
@@ -271,41 +250,6 @@ void _gnutls_free_private_key(gnutls_private_key pkey)
 		_gnutls_mpi_release(&pkey.params[i]);
 	}
 
-	_gnutls_free_datum(&pkey.raw);
-
 	return;
 }
 
-/**
-  * gnutls_x509_extract_key_pk_algorithm - This function returns the keys's PublicKey algorithm
-  * @cert: is a DER encoded private key
-  *
-  * This function will return the public key algorithm of a DER encoded private
-  * key.
-  *
-  * Returns a member of the gnutls_pk_algorithm enumeration on success,
-  * or GNUTLS_E_UNKNOWN_PK_ALGORITHM on error.
-  *
-  **/
-int gnutls_x509_extract_key_pk_algorithm( const gnutls_datum * key)
-{
-int cv, pk;
-
-        pk = GNUTLS_E_UNKNOWN_PK_ALGORITHM;
-
-	/* The only way to distinguish the keys
-	 * is to count the sequence of integers.
-	 */
-	cv = _gnutls_der_check_if_rsa_key( key);
-	if (cv==0)
-		pk = GNUTLS_PK_RSA;
-	else {
-		cv = _gnutls_der_check_if_dsa_key( key);
-		if (cv==0)
-			pk = GNUTLS_PK_DSA;
-	}
-
-	return pk;
-
-}
-
