@@ -26,17 +26,20 @@
 #include <gnutls_int.h>
 #include <gnutls_errors.h>
 #include <gnutls_datum.h>
+#include <gnutls_rsa_export.h>
 #include "debug.h"
 
 /* This function takes a number of bits and returns a supported
  * number of bits. Ie a number of bits that we have a prime in the
  * dh_primes structure.
  */
-static int supported_bits[] = { 512, 0 };
+
+#define MAX_SUPPORTED_BITS 512
+
 static int normalize_bits(int bits)
 {
-	if (bits >= 512)
-		bits = 512;
+	if (bits >= MAX_SUPPORTED_BITS)
+		bits = MAX_SUPPORTED_BITS;
 
 	return bits;
 }
@@ -150,27 +153,12 @@ int _gnutls_rsa_generate_params(GNUTLS_MPI* resarr, int bits)
 
 }
 
-/* returns a negative value if the bits size is not supported 
- */
-static int check_bits(int bits)
-{
-	int i = 0;
-	do {
-		if (supported_bits[i] == bits)
-			return 0;
-		i++;
-	} while (supported_bits[i] != 0);
-
-	gnutls_assert();
-	return GNUTLS_E_INVALID_REQUEST;
-}
-
 #define FREE_PRIVATE_PARAMS for (i=0;i<RSA_PRIVATE_PARAMS;i++) \
 		_gnutls_mpi_release(&rsa_params->params[i])
 
 
 /**
-  * gnutls_rsa_params_set - This function will replace the old RSA parameters
+  * gnutls_rsa_params_import_raw - This function will replace the old RSA parameters
   * @rsa_params: Is a structure will hold the parameters
   * @m: holds the modulus
   * @e: holds the public exponent
@@ -178,29 +166,18 @@ static int check_bits(int bits)
   * @p: holds the first prime (p)
   * @q: holds the second prime (q)
   * @u: holds the coefficient
-  * @bits: is the modulus's number of bits
   *
-  * This function will replace the parameters used in the RSA-EXPORT key
-  * exchange. The new parameters should be stored in the
-  * appropriate gnutls_datum. 
+  * This function will replace the parameters in the given structure.
+  * The new parameters should be stored in the appropriate gnutls_datum. 
   * 
-  * Note that the bits value should only be 512. That is because the
-  * RSA-EXPORT ciphersuites are only allowed to sign a modulus of 512 bits.
-  *
   **/
-int gnutls_rsa_params_set(gnutls_rsa_params rsa_params, 
+int gnutls_rsa_params_import_raw(gnutls_rsa_params rsa_params, 
 	gnutls_datum m, gnutls_datum e,
-	gnutls_datum d, gnutls_datum p, gnutls_datum q, gnutls_datum u,
-	int bits) 
+	gnutls_datum d, gnutls_datum p, gnutls_datum q, gnutls_datum u)
 {
 	int i = 0;
 	size_t siz = 0;
 
-	if (check_bits(bits) < 0) {
-		gnutls_assert();
-		return GNUTLS_E_INVALID_REQUEST;
-	}
-
 	for (i=0;i<RSA_PRIVATE_PARAMS;i++) {
 		_gnutls_mpi_release(&rsa_params->params[i]);
 	}
@@ -292,101 +269,103 @@ int i;
 
 }
 
-#define FREE_ALL_MPIS for (i=0;i<sizeof(rsa_params)/sizeof(GNUTLS_MPI);i++) \
-	_gnutls_mpi_release( &rsa_params[i]) \
+/**
+  * gnutls_rsa_params_generate2 - This function will generate temporary RSA parameters
+  * @params: The structure where the parameters will be stored
+  * @bits: is the prime's number of bits
+  *
+  * This function will generate new temporary RSA parameters for use in 
+  * RSA-EXPORT ciphersuites.  This function is normally slow. 
+  * 
+  * Note that if the parameters are to be used in export cipher suites the 
+  * bits value should be 512 or less.
+  * Also note that the generation of new RSA parameters is only usefull
+  * to servers. Clients use the parameters sent by the server, thus it's
+  * no use calling this in client side.
+  *
+  **/
+int gnutls_rsa_params_generate2(gnutls_rsa_params params, int bits)
+{
+
+	int ret;
+
+	ret = _gnutls_rsa_generate_params( params->params, bits);
+	if (ret < 0) {
+		gnutls_assert();
+		return ret;
+	}
+
+	return 0;
+
+}
 
 /**
-  * gnutls_rsa_params_generate - This function will generate temporary RSA parameters
+  * gnutls_rsa_params_export_raw - This function will export the RSA parameters
+  * @params: a structure that holds the rsa parameters
   * @m: will hold the modulus
   * @e: will hold the public exponent
   * @d: will hold the private exponent
   * @p: will hold the first prime (p)
   * @q: will hold the second prime (q)
   * @u: will hold the coefficient
-  * @bits: is the prime's number of bits
+  * @bits: if non null will hold the prime's number of bits
   *
-  * This function will generate new temporary RSA parameters for use in 
-  * RSA-EXPORT ciphersuites. The new parameters will be allocated using
-  * malloc and will be stored in the appropriate datum.
-  * This function is normally slow. An other function
-  * (gnutls_rsa_params_set()) should be called in order to use the 
-  * generated RSA parameters.
+  * This function will export the RSA parameters found in the given
+  * structure. The new parameters will be allocated using
+  * gnutls_malloc() and will be stored in the appropriate datum.
   * 
-  * Note that the bits value should be 512.
-  * Also note that the generation of new RSA parameters is only usefull
-  * to servers. Clients use the parameters sent by the server, thus it's
-  * no use calling this in client side.
-  *
   **/
-int gnutls_rsa_params_generate(gnutls_datum * m, gnutls_datum *e,
+int gnutls_rsa_params_export_raw(gnutls_rsa_params params,
+	gnutls_datum * m, gnutls_datum *e,
 	gnutls_datum *d, gnutls_datum *p, gnutls_datum* q, 
-	gnutls_datum* u, int bits)
+	gnutls_datum* u, int *bits)
 {
-
-	GNUTLS_MPI rsa_params[RSA_PRIVATE_PARAMS];
 	size_t siz;
-	uint i;
-	int ret;
-
-	if (check_bits(bits) < 0) {
-		gnutls_assert();
-		return GNUTLS_E_INVALID_REQUEST;
-	}
-
-	ret = _gnutls_rsa_generate_params( rsa_params, bits);
-	if (ret < 0) {
-		gnutls_assert();
-		return ret;
-	}
 
 	siz = 0;
-	_gnutls_mpi_print(NULL, &siz, rsa_params[0]);
+	_gnutls_mpi_print(NULL, &siz, params->params[0]);
 
-	m->data = malloc(siz);
+	m->data = gnutls_malloc(siz);
 	if (m->data == NULL) {
-		FREE_ALL_MPIS;
 		return GNUTLS_E_MEMORY_ERROR;
 	}
 
 	m->size = siz;
-	_gnutls_mpi_print( m->data, &siz, rsa_params[0]);
+	_gnutls_mpi_print( m->data, &siz, params->params[0]);
 
 	/* E */
 	siz = 0;
-	_gnutls_mpi_print(NULL, &siz, rsa_params[1]);
+	_gnutls_mpi_print(NULL, &siz, params->params[1]);
 
-	e->data = malloc(siz);
+	e->data = gnutls_malloc(siz);
 	if (e->data == NULL) {
-		FREE_ALL_MPIS;
 		_gnutls_free_datum( m);
 		return GNUTLS_E_MEMORY_ERROR;
 	}
 
 	e->size = siz;
-	_gnutls_mpi_print( e->data, &siz, rsa_params[1]);
+	_gnutls_mpi_print( e->data, &siz, params->params[1]);
 
 	/* D */
 	siz = 0;
-	_gnutls_mpi_print(NULL, &siz, rsa_params[2]);
+	_gnutls_mpi_print(NULL, &siz, params->params[2]);
 
-	d->data = malloc(siz);
+	d->data = gnutls_malloc(siz);
 	if (d->data == NULL) {
-		FREE_ALL_MPIS;
 		_gnutls_free_datum( m);
 		_gnutls_free_datum( e);
 		return GNUTLS_E_MEMORY_ERROR;
 	}
 
 	d->size = siz;
-	_gnutls_mpi_print( d->data, &siz, rsa_params[2]);
+	_gnutls_mpi_print( d->data, &siz, params->params[2]);
 
 	/* P */
 	siz = 0;
-	_gnutls_mpi_print(NULL, &siz, rsa_params[3]);
+	_gnutls_mpi_print(NULL, &siz, params->params[3]);
 
-	p->data = malloc(siz);
+	p->data = gnutls_malloc(siz);
 	if (p->data == NULL) {
-		FREE_ALL_MPIS;
 		_gnutls_free_datum( m);
 		_gnutls_free_datum( e);
 		_gnutls_free_datum( d);
@@ -394,15 +373,14 @@ int gnutls_rsa_params_generate(gnutls_datum * m, gnutls_datum *e,
 	}
 
 	p->size = siz;
-	_gnutls_mpi_print(p->data, &siz, rsa_params[3]);
+	_gnutls_mpi_print(p->data, &siz, params->params[3]);
 
 	/* Q */
 	siz = 0;
-	_gnutls_mpi_print(NULL, &siz, rsa_params[4]);
+	_gnutls_mpi_print(NULL, &siz, params->params[4]);
 
-	q->data = malloc(siz);
+	q->data = gnutls_malloc(siz);
 	if (q->data == NULL) {
-		FREE_ALL_MPIS;
 		_gnutls_free_datum( m);
 		_gnutls_free_datum( e);
 		_gnutls_free_datum( d);
@@ -411,15 +389,14 @@ int gnutls_rsa_params_generate(gnutls_datum * m, gnutls_datum *e,
 	}
 
 	q->size = siz;
-	_gnutls_mpi_print(q->data, &siz, rsa_params[4]);
+	_gnutls_mpi_print(q->data, &siz, params->params[4]);
 
 	/* U */
 	siz = 0;
-	_gnutls_mpi_print(NULL, &siz, rsa_params[5]);
+	_gnutls_mpi_print(NULL, &siz, params->params[5]);
 
-	u->data = malloc(siz);
+	u->data = gnutls_malloc(siz);
 	if (u->data == NULL) {
-		FREE_ALL_MPIS;
 		_gnutls_free_datum( m);
 		_gnutls_free_datum( e);
 		_gnutls_free_datum( d);
@@ -429,13 +406,10 @@ int gnutls_rsa_params_generate(gnutls_datum * m, gnutls_datum *e,
 	}
 
 	u->size = siz;
-	_gnutls_mpi_print(u->data, &siz, rsa_params[5]);
-
-	FREE_ALL_MPIS;
-
-	_gnutls_log("Generated %d bits modulus %s, exponent %s.\n",
-		    bits, _gnutls_bin2hex(m->data, m->size),
-		    _gnutls_bin2hex( e->data, e->size));
+	_gnutls_mpi_print(u->data, &siz, params->params[5]);
+	
+	if (bits)
+		*bits = _gnutls_mpi_get_nbits(params->params[3]);
 
 	return 0;
 
