@@ -32,7 +32,6 @@
 #include "auth_srp.h"
 #include <gnutls_str.h>
 #include <auth_cert.h>
-#include <x509_verify.h>
 #include <gnutls_datum.h>
 #include <gnutls_sig.h>
 #include <auth_srp.h>
@@ -175,7 +174,7 @@ opaque* p;
 		case GNUTLS_CRT_X509:
 			if ((ret =
 			     _gnutls_x509_cert2gnutls_cert( &peer_cert,
-					     info->raw_certificate_list[0], CERT_NO_COPY)) < 0) {
+					     &info->raw_certificate_list[0], CERT_NO_COPY)) < 0) {
 				gnutls_assert();
 				return ret;
 			}
@@ -188,7 +187,7 @@ opaque* p;
 			}
 			if ((ret =
 			     _E_gnutls_openpgp_cert2gnutls_cert( &peer_cert,
-					     info->raw_certificate_list[0])) < 0) {
+					     &info->raw_certificate_list[0])) < 0) {
 				gnutls_assert();
 				return ret;
 			}
@@ -204,7 +203,7 @@ opaque* p;
 				      &peer_cert,
 				      &vparams, &signature);
 	
-	_gnutls_free_cert( peer_cert);
+	_gnutls_free_cert( &peer_cert);
 	if (ret < 0) {
 		gnutls_assert();
 		return ret;
