@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 2000,2001,2002 Nikos Mavroyanopoulos
+ *      Copyright (C) 2000,2001,2002,2003 Nikos Mavroyanopoulos
  *
  * This file is part of GNUTLS.
  *
@@ -29,6 +29,7 @@
 #include <unistd.h>
 #include <gnutls/gnutls.h>
 #include <gnutls/extra.h>
+#include <gnutls/x509.h>
 #include <sys/time.h>
 #include <signal.h>
 #include <netdb.h>
@@ -69,8 +70,9 @@ char *x509_certfile;
 char *x509_cafile;
 char *x509_crlfile = NULL;
 static int x509ctype;
+static int disable_extensions;
 
-static gnutls_srp_client_credentials cred;
+static gnutls_srp_client_credentials srp_cred;
 static gnutls_anon_client_credentials anon_cred;
 static gnutls_certificate_credentials xcred;
 
@@ -108,16 +110,36 @@ void init_global_tls_stuff(void);
 
 
 #define MAX(X,Y) (X >= Y ? X : Y);
-#define DEFAULT_X509_CAFILE "x509/ca.pem"
-#define DEFAULT_X509_KEYFILE2 "x509/clikey-dsa.pem"
-#define DEFAULT_X509_CERTFILE2 "x509/clicert-dsa.pem"
 
-#define DEFAULT_X509_KEYFILE "x509/clikey.pem"
-#define DEFAULT_X509_CERTFILE "x509/clicert.pem"
+/* A callback function to be used at the certificate selection time.
+ */
+static int cert_callback( gnutls_session session, const gnutls_datum* client_certs,
+	int client_certs_num, const gnutls_datum * req_ca_rdn, int nreqs)
+{
+char issuer_dn[256];
+int len, i, ret;
+
+	/* Print the server's trusted CAs
+	 */
+	if (nreqs > 0)
+		printf("- Server's trusted authorities:\n");
+	else
+		printf("- Server did not send us any trusted authorities names.\n");
+
+	/* print the names (if any) */
+	for (i=0;i<nreqs;i++) {
+		len = sizeof(issuer_dn);
+		ret = gnutls_x509_rdn_get( &req_ca_rdn[i], issuer_dn, &len);
+		if ( ret >= 0) {
+			printf("   [%d]: ", i);
+			printf("%s\n", issuer_dn);
+		}
+	}
+
+	return -1;
+
+}
 
-#define DEFAULT_PGP_KEYFILE "openpgp/cli_sec.asc"
-#define DEFAULT_PGP_CERTFILE "openpgp/cli_pub.asc"
-#define DEFAULT_PGP_KEYRING "openpgp/cli_ring.gpg"
 
 /* initializes a gnutls_session with some defaults.
  */
@@ -129,9 +151,11 @@ static gnutls_session init_tls_session( const char* hostname)
 
    /* allow the use of private ciphersuites.
     */
-   gnutls_handshake_set_private_extensions(session, 1);
+   if (disable_extensions == 0)
+      gnutls_handshake_set_private_extensions(session, 1);
 
-   gnutls_set_server_name( session, GNUTLS_NAME_DNS, hostname, strlen(hostname));
+   if (disable_extensions == 0)
+      gnutls_set_server_name( session, GNUTLS_NAME_DNS, hostname, strlen(hostname));
 
    gnutls_cipher_set_priority(session, cipher_priority);
    gnutls_compression_set_priority(session, comp_priority);
@@ -143,16 +167,17 @@ static gnutls_session init_tls_session( const char* hostname)
    gnutls_dh_set_prime_bits(session, 512);
 
    gnutls_credentials_set(session, GNUTLS_CRD_ANON, anon_cred);
-   if (srp_username != NULL)
-      gnutls_credentials_set(session, GNUTLS_CRD_SRP, cred);
+   gnutls_credentials_set(session, GNUTLS_CRD_SRP, srp_cred);
    gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, xcred);
 
+   gnutls_certificate_client_set_select_function( session, cert_callback);
+
    /* send the fingerprint */
    if (fingerprint != 0)
       gnutls_openpgp_send_key(session, GNUTLS_OPENPGP_KEY_FINGERPRINT);
 
    /* use the max record size extension */
-   if (record_max_size > 0) {
+   if (record_max_size > 0 && disable_extensions == 0) {
       if (gnutls_record_set_max_size(session, record_max_size) < 0) {
 	 fprintf(stderr,
 		 "Cannot set the maximum record size to %d.\n",
@@ -362,8 +387,7 @@ int main(int argc, char **argv)
       socket_bye(&hd);
 
 
-   if (srp_username != NULL)
-      gnutls_srp_free_client_credentials(cred);
+   gnutls_srp_free_client_credentials(srp_cred);
    gnutls_certificate_free_credentials(xcred);
    gnutls_anon_free_client_credentials(anon_cred);
 
@@ -372,19 +396,16 @@ int main(int argc, char **argv)
    return 0;
 }
 
-#undef DEBUG
-
 static gaainfo info;
 void gaa_parser(int argc, char **argv)
 {
-   int i, j;
-
    if (gaa(argc, argv, &info) != -1) {
       fprintf(stderr,
 	      "Error in the arguments. Use the --help or -h parameters to get more information.\n");
       exit(1);
    }
 
+   disable_extensions = info.disable_extensions;
    xml = info.xml;
    starttls = info.starttls;
    resume = info.resume;
@@ -397,48 +418,15 @@ void gaa_parser(int argc, char **argv)
    else
       x509ctype = GNUTLS_X509_FMT_DER;
 
-#ifdef DEBUG
-   if (info.x509_certfile != NULL)
-      x509_certfile = info.x509_certfile;
-   else
-      x509_certfile = DEFAULT_X509_CERTFILE;
-
-   if (info.x509_keyfile != NULL)
-      x509_keyfile = info.x509_keyfile;
-   else
-      x509_keyfile = DEFAULT_X509_KEYFILE;
-
-   if (info.x509_cafile != NULL)
-      x509_cafile = info.x509_certfile;
-   else
-      x509_cafile = DEFAULT_X509_CAFILE;
-
-   if (info.pgp_certfile != NULL)
-      pgp_certfile = info.pgp_certfile;
-   else
-      pgp_certfile = DEFAULT_PGP_CERTFILE;
-
-   if (info.pgp_keyfile != NULL)
-      pgp_keyfile = info.pgp_keyfile;
-   else
-      pgp_keyfile = DEFAULT_PGP_KEYFILE;
-
-   if (info.srp_passwd != NULL)
-      srp_passwd = info.srp_passwd;
-
-   if (info.srp_username != NULL)
-      srp_username = info.srp_username;
-#else
    srp_username = info.srp_username;
    srp_passwd = info.srp_passwd;
    x509_cafile = info.x509_cafile;
+   x509_crlfile = info.x509_crlfile;
    x509_keyfile = info.x509_keyfile;
    x509_certfile = info.x509_certfile;
    pgp_keyfile = info.pgp_keyfile;
    pgp_certfile = info.pgp_certfile;
 
-#endif
-
    pgp_keyring = info.pgp_keyring;
    pgp_trustdb = info.pgp_trustdb;
 
@@ -462,7 +450,7 @@ void gaa_parser(int argc, char **argv)
 void cli_version(void)
 {
    fprintf(stderr, "GNU TLS test client, ");
-   fprintf(stderr, "version %s.\n", LIBGNUTLS_VERSION);
+   fprintf(stderr, "version %s. Libgnutls %s.\n", LIBGNUTLS_VERSION, gnutls_check_version(NULL));
 }
 
 
@@ -570,6 +558,12 @@ int do_handshake(socket_st* socket)
    return ret;
 }
 
+
+static void tls_log_func( const char* str)
+{
+	fprintf(stderr, "|** %s", str);
+}
+
 void init_global_tls_stuff()
 {
 int ret;
@@ -578,6 +572,7 @@ int ret;
       fprintf(stderr, "global state initialization error\n");
       exit(1);
    }
+   gnutls_global_set_log_function( tls_log_func);
 
    if (gnutls_global_init_extra() < 0) {
       fprintf(stderr, "global state (extra) initialization error\n");
@@ -590,6 +585,10 @@ int ret;
       exit(1);
    }
 
+   /* there are some intermediate CAs that have a v1 certificate *%&@#*%&
+    */
+   gnutls_certificate_set_verify_flags( xcred, GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT);
+
    if (x509_cafile != NULL) {
       ret =
 	  gnutls_certificate_set_x509_trust_file(xcred,
@@ -601,6 +600,17 @@ int ret;
       }
    }
 
+   if (x509_crlfile != NULL) {
+      ret =
+	  gnutls_certificate_set_x509_crl_file(xcred,
+						 x509_crlfile, x509ctype);
+      if (ret < 0) {
+	 fprintf(stderr, "Error setting the x509 CRL file\n");
+      } else {
+	 printf("Processed %d CRL(s).\n", ret);
+      }
+   }
+
    if (x509_certfile != NULL) {
       ret =
 	  gnutls_certificate_set_x509_key_file(xcred,
@@ -639,15 +649,15 @@ int ret;
 	 fprintf(stderr, "Error setting the OpenPGP trustdb file\n");
       }
    }
-/*	gnutls_certificate_client_callback_func( xcred, cert_callback); */
+
 
    /* SRP stuff */
-   if (srp_username != NULL) {
-      if (gnutls_srp_allocate_client_credentials(&cred) < 0) {
-	 fprintf(stderr, "SRP authentication error\n");
-      }
+   if (gnutls_srp_allocate_client_credentials(&srp_cred) < 0) {
+	fprintf(stderr, "SRP authentication error\n");
+   }
 
-      if ((ret=gnutls_srp_set_client_credentials(cred, srp_username, srp_passwd)) < 0) {
+   if (srp_username != NULL) {
+      if ((ret=gnutls_srp_set_client_credentials(srp_cred, srp_username, srp_passwd)) < 0) {
 	 fprintf(stderr, "SRP credentials set error [%d]\n", ret);
       }
    }
