@@ -8,50 +8,54 @@ typedef struct _gaainfo gaainfo;
 
 struct _gaainfo
 {
-#line 80 "serv.gaa"
+#line 86 "serv.gaa"
 	char **ctype;
-#line 79 "serv.gaa"
+#line 85 "serv.gaa"
 	int nctype;
-#line 76 "serv.gaa"
+#line 82 "serv.gaa"
 	char **kx;
-#line 75 "serv.gaa"
+#line 81 "serv.gaa"
 	int nkx;
-#line 72 "serv.gaa"
+#line 78 "serv.gaa"
 	char **macs;
-#line 71 "serv.gaa"
+#line 77 "serv.gaa"
 	int nmacs;
-#line 68 "serv.gaa"
+#line 74 "serv.gaa"
 	char **comp;
-#line 67 "serv.gaa"
+#line 73 "serv.gaa"
 	int ncomp;
-#line 64 "serv.gaa"
+#line 70 "serv.gaa"
 	char **proto;
-#line 63 "serv.gaa"
+#line 69 "serv.gaa"
 	int nproto;
-#line 60 "serv.gaa"
+#line 66 "serv.gaa"
 	char **ciphers;
-#line 59 "serv.gaa"
+#line 65 "serv.gaa"
 	int nciphers;
-#line 55 "serv.gaa"
+#line 61 "serv.gaa"
 	char *srp_passwd_conf;
-#line 52 "serv.gaa"
+#line 58 "serv.gaa"
 	char *srp_passwd;
-#line 49 "serv.gaa"
+#line 55 "serv.gaa"
 	char *x509_certfile;
-#line 46 "serv.gaa"
+#line 52 "serv.gaa"
 	char *x509_keyfile;
-#line 43 "serv.gaa"
+#line 49 "serv.gaa"
 	char *pgp_certfile;
-#line 40 "serv.gaa"
+#line 46 "serv.gaa"
 	char *pgp_keyfile;
-#line 37 "serv.gaa"
+#line 43 "serv.gaa"
 	char *pgp_trustdb;
-#line 34 "serv.gaa"
+#line 40 "serv.gaa"
 	char *pgp_keyring;
-#line 31 "serv.gaa"
+#line 37 "serv.gaa"
+	char *x509_crlfile;
+#line 34 "serv.gaa"
 	char *x509_cafile;
-#line 28 "serv.gaa"
+#line 31 "serv.gaa"
 	int fmtder;
+#line 28 "serv.gaa"
+	char *dh_params_file;
 #line 24 "serv.gaa"
 	int http;
 #line 21 "serv.gaa"
