@@ -1,5 +1,5 @@
 /*
- *      Copyright (C) 2000,2001,2002 Nikos Mavroyanopoulos
+ * Copyright (C) 2000,2001,2002,2003 Nikos Mavroyanopoulos
  *
  * This file is part of GNUTLS.
  *
@@ -20,6 +20,7 @@
 
 #include <gnutls/gnutls.h>
 #include <gnutls/extra.h>
+#include <gnutls/x509.h>
 #include <tests.h>
 #include <unistd.h>
 #include <signal.h>
@@ -33,6 +34,7 @@ extern gnutls_certificate_credentials xcred;
 
 extern int more_info;
 static int srp = 0;
+static int dh_bits;
 
 extern int tls1_ok;
 extern int ssl3_ok;
@@ -183,29 +185,44 @@ int ret;
 }
 
 int test_export( gnutls_session session) {
-		ADD_ALL_COMP(session);
-		ADD_ALL_CERTTYPES(session);
-		ADD_ALL_PROTOCOLS(session);
-		ADD_ALL_MACS(session);
+	ADD_ALL_COMP(session);
+	ADD_ALL_CERTTYPES(session);
+	ADD_ALL_PROTOCOLS(session);
+	ADD_ALL_MACS(session);
 
-		ADD_KX(session, GNUTLS_KX_RSA_EXPORT);
-		ADD_CIPHER(session, GNUTLS_CIPHER_ARCFOUR_40);
-		gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, xcred);
+	ADD_KX(session, GNUTLS_KX_RSA_EXPORT);
+	ADD_CIPHER(session, GNUTLS_CIPHER_ARCFOUR_40);
+	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, xcred);
 
-		return do_handshake( session);
+	return do_handshake( session);
 }
 
 int test_dhe( gnutls_session session) {
-		ADD_ALL_CIPHERS(session);
-		ADD_ALL_COMP(session);
-		ADD_ALL_CERTTYPES(session);
-		ADD_ALL_PROTOCOLS(session);
-		ADD_ALL_MACS(session);
+int ret;
 
-		ADD_KX2(session, GNUTLS_KX_DHE_RSA, GNUTLS_KX_DHE_DSS);
-		gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, xcred);
+	ADD_ALL_CIPHERS(session);
+	ADD_ALL_COMP(session);
+	ADD_ALL_CERTTYPES(session);
+	ADD_ALL_PROTOCOLS(session);
+	ADD_ALL_MACS(session);
 
-		return do_handshake( session);
+	ADD_KX2(session, GNUTLS_KX_DHE_RSA, GNUTLS_KX_DHE_DSS);
+	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, xcred);
+
+	ret = do_handshake( session);
+	dh_bits = gnutls_dh_get_prime_bits( session);
+	if (dh_bits < 0) dh_bits = 0;
+
+	return ret;
+}
+
+int test_dhe_bits( gnutls_session session) {
+int ret;
+
+	if (dh_bits == 0) return FAILED;
+
+	printf( " %d", dh_bits);
+	return SUCCEED;
 }
 
 int test_ssl3( gnutls_session session) {
@@ -386,6 +403,9 @@ int ret;
 
 }
 
+/* Advertize both TLS 1.0 and SSL 3.0 if the connection fails,
+ * but the previous SSL 3.0 test succeeded then disable TLS 1.0.
+ */
 int test_tls1_2( gnutls_session session) {
 int ret;
 	ADD_ALL_CIPHERS(session);
@@ -399,7 +419,7 @@ int ret;
 	ret = do_handshake( session);
 	if (ret==FAILED) {
 		/* disable TLS 1.0 */
-		if (tls1_ok!=0) {
+		if (ssl3_ok!=0) {
 			protocol_priority[0] = GNUTLS_SSL3;
 			protocol_priority[1] = 0;
 		}
@@ -425,8 +445,8 @@ int ret;
 	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, xcred);
 
 	ret = do_handshake( session);
-	if (ret < 0) return FAILED;
-	
+	if (ret == FAILED) return FAILED;
+
 	if (gnutls_protocol_get_version(session)==GNUTLS_TLS1) return SUCCEED;
 	return UNSURE;
 }
@@ -443,7 +463,7 @@ int ret;
 	gnutls_record_set_max_size( session, 512);
 
 	ret = do_handshake( session);
-	if (ret<0) return FAILED;
+	if (ret == FAILED) return ret;
 
 	ret = gnutls_record_get_max_size(session);
 	if (ret==512) return SUCCEED;
@@ -489,7 +509,7 @@ int ret;
 
 	ret = do_handshake( session);
 	if (ret!=SUCCEED) return ret;
-	
+
 	if (tls1_ok!=0 && gnutls_protocol_get_version( session)==GNUTLS_SSL3)
 		return FAILED;
 	
@@ -498,6 +518,8 @@ int ret;
 
 
 int test_anonymous( gnutls_session session) {
+int ret;
+
 	ADD_ALL_CIPHERS(session);
 	ADD_ALL_COMP(session);
 	ADD_ALL_CERTTYPES(session);
@@ -506,8 +528,11 @@ int test_anonymous( gnutls_session session) {
 	ADD_KX(session, GNUTLS_KX_ANON_DH);
 	gnutls_credentials_set(session, GNUTLS_CRD_ANON, anon_cred);
 
-	return do_handshake( session);
+	ret = do_handshake( session);
+	dh_bits = gnutls_dh_get_prime_bits( session);
+	if (dh_bits < 0) dh_bits = 0;
 
+	return ret;
 }
 
 
@@ -532,7 +557,7 @@ int tmp_session_id_size;
 	tmp_session_id_size = session_id_size;
 
 	ret = do_handshake( session);
-	if (ret < 0) return FAILED;
+	if (ret == FAILED) return ret;
 	
 	/* check if we actually resumed the previous session */
 
@@ -559,10 +584,66 @@ int ret;
 	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, xcred);
 
 	ret = do_handshake( session);
-	if (ret < 0) return FAILED;
+	if (ret == FAILED) return ret;
 
 	printf("\n");
 	print_cert_info( session);
 
 	return SUCCEED;
 }
+
+/* A callback function to be used at the certificate selection time.
+ */
+static int cert_callback( gnutls_session session, const gnutls_datum* client_certs,
+	int client_certs_num, const gnutls_datum * req_ca_rdn, int nreqs)
+{
+char issuer_dn[256];
+int len, i, ret;
+
+	/* Print the server's trusted CAs
+	 */
+	printf("\n");
+	if (nreqs > 0)
+		printf("- Server's trusted authorities:\n");
+	else
+		printf("- Server did not send us any trusted authorities names.\n");
+
+	/* print the names (if any) */
+	for (i=0;i<nreqs;i++) {
+		len = sizeof(issuer_dn);
+		ret = gnutls_x509_rdn_get( &req_ca_rdn[i], issuer_dn, &len);
+		if ( ret >= 0) {
+			printf("   [%d]: ", i);
+			printf("%s\n", issuer_dn);
+		}
+	}
+
+	return -1;
+
+}
+
+/* Prints the trusted server's CAs. This is only
+ * if the server sends a certificate request packet.
+ */
+int test_server_cas( gnutls_session session) 
+{
+int ret;
+
+	ADD_ALL_CIPHERS(session);
+	ADD_ALL_COMP(session);
+	ADD_ALL_CERTTYPES(session);
+	ADD_ALL_PROTOCOLS(session);
+	ADD_ALL_MACS(session);
+	ADD_ALL_KX(session);
+
+	gnutls_credentials_set(session, GNUTLS_CRD_CERTIFICATE, xcred);
+        gnutls_certificate_client_set_select_function( session, cert_callback);
+
+	ret = do_handshake( session);
+	if (ret ==FAILED) return ret;
+
+	return SUCCEED;
+}
+
+
+
